//---------------------------------------------------------------------------
// (c)Alex A. Trubetskoy            alextr438@yandex.ru
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "ListViewFiles.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//
static inline void ValidCtrCheck(TListViewFiles *)
{
    new TListViewFiles(NULL);
}
//---------------------------------------------------------------------------
namespace Listviewfiles
{
    void __fastcall PACKAGE Register()
    {
        TComponentClass classes[1] = {__classid(TListViewFiles)};
        RegisterComponents("Samples", classes, 0);
    }
}
//---------------------------------------------------------------------------
__fastcall TListViewFiles::TListViewFiles(TComponent* Owner)
    : TListView(Owner)
{
    fDir = "c:\\";
    fDrive = 'c';
    this->Parent = (TWinControl*)Owner;   
    this->ViewStyle = vsReport;
    this->SortType = 2;

    TListColumn* listColumn;

    pImageList = new TImageList(this);
    this->SmallImages = pImageList;

    listColumn = this->Columns->Add();
    listColumn->Caption = "Name";
    listColumn->Width = 150;
    listColumn = this->Columns->Add();
    listColumn->Caption = "Ext";
    listColumn->Width = 50;
    listColumn = this->Columns->Add();
    listColumn->Caption = "Size";
    listColumn->Width = 50;
    listColumn = this->Columns->Add();
    listColumn->Caption = "Date";
    listColumn->Width = 100;

    searchFiles();
}
//---------------------------------------------------------------------------
__fastcall TListViewFiles::~TListViewFiles()
{
    delete pImageList;
}
//---------------------------------------------------------------------------
//search all files in the current directory to show them on this control
void __fastcall TListViewFiles::searchFiles()
{
    AnsiString pathMask = fDir + "*.*";
    TSearchRec srec;

    //clear ListView and ImageList
    this->Items->Clear();
    pImageList->Clear();

    if(FindFirst(pathMask, faAnyFile, srec))
        return;

    if(srec.Name != ".") {
        addItem(srec);
    }

    while(!FindNext(srec)) {
        addItem(srec);
    }
    FindClose(srec);
}
//---------------------------------------------------------------------------
void __fastcall TListViewFiles::addItem(TSearchRec& srec)
{
    TIcon* pIcon = new TIcon();
    TPicture* pPic = new TPicture();
    TListItem* listItem;
    DWORD attr;
    SHFILEINFO sfi;
    HICON hIcon;
    int index, point;
    AnsiString fileName, ext;
    try {
        SHGetFileInfo((fDir + srec.Name).c_str(), attr, &sfi, sizeof(SHFILEINFO),
                SHGFI_ICON | SHGFI_SMALLICON);
        hIcon = sfi.hIcon;
        pIcon->Handle = hIcon;
        pPic->Icon = pIcon;
        index = pImageList->AddIcon(pIcon);
        listItem = this->Items->Add();
        if((srec.Attr & faDirectory) > 0) {
            listItem->Caption = " [" + srec.Name + "]";
            listItem->ImageIndex = index;
            listItem->SubItems->Add("");
            listItem->SubItems->Add("<DIR>");
            listItem->SubItems->Add(DateToStr(FileDateToDateTime(srec.Time)));
        }
        else {
            point = srec.Name.LastDelimiter("." );
            if(point > 0) {
                fileName = srec.Name.SubString(1, point - 1);
                ext = srec.Name.SubString(point + 1, srec.Name.Length() - point);
            }
            else {
                fileName = srec.Name;
                ext = "";
            }
            listItem->Caption = fileName;
            listItem->SubItems->Add(ext);
            listItem->SubItems->Add(AnsiString(srec.Size));
            listItem->SubItems->Add(DateToStr(FileDateToDateTime(srec.Time)));
            listItem->ImageIndex = index;
        }
    }
    __finally {
        delete pIcon;
        delete pPic;
    }
}
//---------------------------------------------------------------------------
// set the current directory
void __fastcall TListViewFiles::setDir(AnsiString dir)
{
    if(dir[dir.Length()] != '\\')
        dir += "\\";
    fDir = dir;
    fDrive = fDir[1];
    searchFiles();
}
//---------------------------------------------------------------------------
// return the current directory
AnsiString __fastcall TListViewFiles::getDir()const
{
    return fDir;
}
//---------------------------------------------------------------------------
// set the current drive
void __fastcall TListViewFiles::setDrive(char drive)
{
    fDrive = drive;
    fDir = AnsiString(drive) + ":\\";
    searchFiles();
}
//---------------------------------------------------------------------------
// return the current drive
char __fastcall TListViewFiles::getDrive()const
{
    return fDrive;
}
//---------------------------------------------------------------------------
// return the path and the name of the selected item
AnsiString __fastcall TListViewFiles::getFilePath()const
{
    return (fDir + getFileName());
}
//---------------------------------------------------------------------------
// return only the name of the selected item
AnsiString __fastcall TListViewFiles::getFileName()const
{
    return getNameOfItem(Selected);
}
//---------------------------------------------------------------------------
void __fastcall TListViewFiles::UpdateData()
{
    searchFiles();
}
//---------------------------------------------------------------------------
// determine if the selected item is directory
bool __fastcall TListViewFiles::isDirSel()const
{
    if(Selected != NULL)
        return isDirectory(fDir + Selected->Caption);
    return false;
}
//---------------------------------------------------------------------------
// determine if the item is directory
bool __fastcall TListViewFiles::isDirectory(AnsiString name)const
{
    TSearchRec srec;
    if(FindFirst(name, faAnyFile, srec) == 0) {
        if(srec.Attr & faDirectory) {
            FindClose(srec);
            return true;
        }
        FindClose(srec);
    }
    else {
        name = getDirPath(name);
        if(FindFirst(name, faAnyFile, srec) == 0) {
            if(srec.Attr & faDirectory) {
                FindClose(srec);
                return true;
            }
            FindClose(srec);
        }
    }
    return false;
}
//---------------------------------------------------------------------------
// return the directory's name (without [ and ])
AnsiString TListViewFiles::getDirPath(AnsiString str)const
{
    AnsiString res, temp = "";
    if(str.Length()) {
        int i = str.Length();
        if(str[i] == '\\')
            --i;
        i = str.LastDelimiter("\\");
        if((i > 0) && (str[str.Length()] == ']')) {
            res = str.SubString(1, i);
            temp = str.SubString(i + 1, str.Length() - i);
            temp.Delete(temp.Length(), 1);
            temp = temp.TrimLeft();
            temp = temp.Delete(1, 1);
        }
    }
    return (res + temp);
}
//---------------------------------------------------------------------------
// get the path of the item
AnsiString TListViewFiles::getPathOfItem(TListItem* pItem)const
{
    return (fDir + getNameOfItem(pItem));
}
//---------------------------------------------------------------------------
// get the name of the item
AnsiString TListViewFiles::getNameOfItem(TListItem* pItem)const
{
    AnsiString nameItem = "";
    if(pItem != NULL) {
        if(isDirectory(fDir + pItem->Caption)) {
            nameItem = getDirPath(fDir + pItem->Caption);
            int i = nameItem.Length();
            for(; i > 0; --i)
                if(nameItem[i] == '\\')break;
            nameItem = nameItem.SubString(i + 1, nameItem.Length() - i);
        }
        else {
            nameItem = pItem->Caption;
            if((pItem->SubItems->Strings[0]).Length())
                nameItem += ("." + pItem->SubItems->Strings[0]);
        }
    }
    return nameItem;
}
//---------------------------------------------------------------------------

